#! /bin/perl

srand (time ^ $$ ^ unpack "%L*", `ps axww | gzip -f`);

$volInSyringe=100.0;
$simTimeStep=1.0;
$maxFaults=0.0;
$minFaultTime=int(rand(4.0));
$maxFaultTime=int(5.0 + rand(5.0));

$drugConc = 20.0; # mcg/ml
$pWeight = 20.0 + rand(90.0); # kgs
$contDose = 0.01 + rand(0.5); # mcg/kg/min 
$pcaDose = 0.05 + rand(0.5); # mcg/kg
$pcaDeliveryTime = 2; # min
$pcaLockout = int(3 + rand(12.0)); # min
$pcaLimit = int(3 + rand(9)); #max req/hr


$cRate = $contDose * $pWeight/$drugConc; # ml/min
$pRate = $pcaDose * $pWeight/($drugConc * $pcaDeliveryTime); # ml/min


sub printParam{
    my($parName,$parVal,$parComment) = @_;
    printf  OFILE "param <sim_param> %s: num EQ %.2f; # %s \n", $parName, $parVal, $parComment;
}

sub printStem{
    while ($line= <STEMFILE>){
	printf OFILE "%s",$line;
    }
}

sub printHeader{
    printParam ("volInSyringe", $volInSyringe, "#ml");
    printParam ("drugConc", $drugConc, "#mcg/ml");
    printParam ("simTimeStep", $simTimeStep ,"#min");
    printParam ("maxFaults", $maxFaults, "# <number> " );
    printParam ("minFaultTime", $minFaultTime, "# min ");
    printParam ("maxFaultTime", $maxFaultTime, "# min ");
    printParam ("cRate", $cRate, "#ml/min");
    printParam ("pcaBolusRate", $pRate, "#ml/min");
    printParam ("pcaBolusTime", $pcaDeliveryTime, "#min");
    printParam ("pcaLockout", $pcaLockout, "#min");
    printParam ("pcaLimit", $pcaLimit, "# <number> ");
}

sub printOutput {
    my($index,$descr) = @_;
    $outFileName = sprintf("prescript-%d-%s.aut", $index, $descr);
    printf RUNSCRIPT "time ../../autoParser -d 25 %s >& output-%d-%s  ; \n", $outFileName,$index,$descr; 
    open STEMFILE, "<pcaMode.stem" or die $!;
    open OFILE, ">".$outFileName or die $!;
    print $outFileName;
    printHeader;
    printStem;
    close STEMFILE;
    close OFILE;
}

open RUNSCRIPT, "> runPrescript.sh" or die $!;
printf RUNSCRIPT "#! /bin/bash\n";

## ORIGINAL
printOutput(0,"original");
$maxFaults=1.0;
printOutput(1,"original");
$maxFaults=2.0;
printOutput(2,"original");
$maxFaults=4.0;
printOutput(4,"original");

##MODE SELECTION ERROR

$cRateOld = $cRate;
$pRateOld = $pRate;
$cRate = 0.0;
printOutput(0,"modeSelectionError");
$maxFaults=1.0;
printOutput(1,"modeSelectionError");
$maxFaults=2.0;
printOutput(2,"modeSelectionError");
$maxFaults=4.0;
printOutput(4,"modeSelectionError");
$cRate= $cRateOld;

##UNIT ERROR

$cRate = $cRate * 60.0;
printOutput(0,"unitError");
$maxFaults=1.0;
printOutput(1,"unitError");
$maxFaults=2.0;
printOutput(2,"unitError");
$maxFaults=4.0;
printOutput(4,"unitError");
$cRate = $cRateOld;

## LBS as kgs
$cRate =$cRate * 2.2;
$pRate =$pRate * 2.2;
printOutput(0,"LbError");
$maxFaults=1.0;
printOutput(1,"LbError");
$maxFaults=2.0;
printOutput(2,"LbError");
$maxFaults=4.0;
printOutput(4,"LbError");
$cRate = $cRateOld;
$pRate = $pRateOld;

#kgs as LBs

## LBS as kgs
$cRate =$cRate / 2.2;
$pRate =$pRate / 2.2;
printOutput(0,"KgError");
$maxFaults=1.0;
printOutput(1,"KgError");
$maxFaults=2.0;
printOutput(2,"KgError");
$maxFaults=4.0;
printOutput(4,"KgError");
$cRate = $cRateOld;
$pRate = $pRateOld;

close RUNSCRIPT;
