# /usr/bin/perl 

# Process CSV files to generate parameter sets

# Inputs: 
#   1. Core set of parameters to be used
#   2. CSV file
#   Expected CSV file format
#   1. Header row for file
#   <Num>, <ERR_ID> , Rate (ml/hr), Volume (ml), Conc(mcg), Dose, Weight, Time, Discarded?
#   We will keep reading until we get to the first blank line


$inpFile = "simulation_5.csv";
$outStem = "sim_5_";
$rowCount=1;

if ($#ARGV < 0 || $#ARGV > 1){
    printf "Usage: processCSVFiles fileStem (note .csv extension will be assumed ) \n";
    exit 1
}


$inpFile=$ARGV[0].".csv";
$outStem=$ARGV[0];






open INPFILE, "<".$inpFile or die ("Cannot open input file:".$inpFile) ;
open FILESTORUN, ">> paramsToRun" or die "No write permissions. \n";
@rowHeaders=();

sub exportToFile {
    my(@rowFields) = @_;
    $rowID=$rowFields[0];
    $rowErrID=$rowFields[1];
    $fileName=sprintf("%s_%s_%s.params", $outStem, $rowID, $rowErrID);
    open OFILE, ">".$fileName or die $!;
    printf OFILE "contRate %f\n", $rowFields[2]/60.0; #Update units to ml/min
    printf OFILE "volInSyringe %f\n", $rowFields[3];
    printf OFILE "drugConc %f\n", $rowFields[4];
    close OFILE;
    printf "Printed row id # %d in file %s \n", $rowID, $fileName;
    printf FILESTORUN "%s_%s_%s\n", $outStem, $rowID, $rowErrID;
}

while ($line = <INPFILE>){
    chomp $line;
    @rowFields = split(/\s*,\s*/,$line);

    if ($line eq "" || $#rowFields < 0){
	last;
    } else {

	printf "-->%s<<\n", (join(":",@rowFields));
    
	if ($rowCount == 1){
	    @rowHeaders=@rowFields;
	} else {
	    $lastRow = $rowFields[-1];
	    if ($lastRow =~ m/^DISCARDED\_*/){
		printf "Row %d discarded. \n", $rowCount-1;
	    } else {
		exportToFile (@rowFields);
	    }
	}
    }
    
    $rowCount = $rowCount+1;

}

 
close FILESTORUN;
close INPFILE;



