{ 

  module P=Printf 

  let lineNum = ref 1

  let update_position lex_buf =
    let pos = lex_buf.Lexing.lex_curr_p in
      lex_buf.Lexing.lex_curr_p <- { pos with
				       Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
				       Lexing.pos_bol = pos.Lexing.pos_cnum 
				   };
      incr lineNum 
	
  open AutoParser

}

let digit = '-'?['0'-'9']
let id = ['a'-'z' 'A'-'Z' '_' ]['a'-'z' 'A'-'Z' '_' '0'-'9']*

rule autoLexer  = parse 
  | "/*" { longComments lexbuf }
  | "//" { lineComments lexbuf }
  | "#"  { lineComments lexbuf }
  | (digit+) ('.' (digit+))? as dStr { TNum ( dStr) }
  | "true" {TBool true}
  | "false" {TBool false }
  | '\"'([^'\"']* as dStr)'\"' { TString dStr }
  | "num" {TKwdNum}
  | "bool" {TKwdBool}
  | "boolean" {TKwdBool}
  | "init" {TKwdInit}
  | "string" {TKwdString}
  | "IN" {TKwdIn}
  | "in" {TKwdIn}
  | "eq" {TKwdEq}
  | "when" {TKwdWhen}
  | "EQ" {TKwdEq}
  | "WHERE" {TKwdWhere}
  | "where" {TKwdWhere}
  | "input" {TKwdInput }
  | "|" { TVert }
  | "param" {TKwdParam }
  | "user_input" {TKwdUserInput}
  | "sim_param" {TKwdSimParam }
  | ":" {TColon}
  | "enum" {TKwdEnum}
  | "unit" {TKwdUnit}
  | "assert" {TKwdAssert}
  | "assume" {TKwdAssume}
  | "defined" {TKwdDefined}
  | "clock" {TKwdClock}
  | "freeze" {TKwdFreeze}
  | "unfreeze" {TKwdUnFreeze}
  | "var" {TKwdVar}
  | "nondet" {TKwdNondet}
  | "component" {TKwdComponent}
  | "." {TKwdDot}
  | ":= " {TAssign }
  | "+" { TPlus }
  | "-" {TMinus }
  | "/" {TDiv }
  | "*" {TStar }
  | "==" {TEq}
  | ";" {TSemiColon}
  | "," {TComma}
  | "&&" {TAnd }
  | "||" {TOr }
  | "=>" { TImpl }
  | "->" { TArrow}
  | "!" {TNot}
  | "(" {TOpenRound  }
  | ")" {TCloseRound  }
  | "{" {TOpenCurly  }
  | "}" {TCloseCurly }
  | "[" {TOpenSquare }
  | "]" {TCloseSquare}
  | "<" {TLt }
  | "<=" {TLeq }
  | ">" {TGt}
  | ">=" {TGeq}
  | "!=" {TNeq }
  | ['\t' ' '] { autoLexer lexbuf }
  | ['\n'] { update_position lexbuf; autoLexer lexbuf }
  | "end" {TKwdEnd}
  | "step" {TKwdStep}
  | "max" {TKwdMax}
  | eof { TKwdEnd}
  | id as iStr { TName iStr }
  | _ as wStr { 
      ignore (
	P.fprintf stderr "Unknown character %c @ line : %d " wStr 
	  lexbuf.Lexing.lex_curr_p.Lexing.pos_lnum ) ;
      autoLexer lexbuf 
    } 

and longComments  = parse 
  | '\n' { update_position lexbuf; longComments lexbuf }
  | "*/" { autoLexer lexbuf }
  | _ { longComments lexbuf }
and lineComments = parse
  | '\n' {update_position lexbuf; autoLexer lexbuf }
  | _ { lineComments lexbuf }

		
