%{
  module P=Printf
  open MyUtils
  open CompAst
  let parser_err msg = P.fprintf stderr "Error: %s \n" msg


  let printMessage k msghdr msg = 
    let start_pos = Parsing.rhs_start_pos k in
    let end_pos = Parsing.rhs_end_pos k in
      ignore (P.fprintf stderr  "%s- %d.%d-%d.%d: %s\n "
		msghdr
		start_pos.Lexing.pos_lnum 
		(start_pos.Lexing.pos_cnum - start_pos.Lexing.pos_bol)
		end_pos.Lexing.pos_lnum 
		(end_pos.Lexing.pos_cnum - end_pos.Lexing.pos_bol) msg )
    
  let printError k msg = 
    printMessage k "Error:" msg;
    raise Parsing.Parse_error

  let warn k msg = 
    printMessage k "Warning:" msg
  
  let relOp op p1 p2 = 
    let (e1,t1) = p1 in 
    let (e2,t2) = p2 in
    match (t1,t2) with
      | RealType, RealType -> 
	( ARelOp (op, e1, e2) ), BoolType
      | _ -> 
	printError 2 " relational operator type mismatched"
	
  let boolOp op p1 p2 = 
    let (e1,t1) = p1 in 
    let (e2,t2) = p2 in
    match (t1,t2) with 
      | BoolType, BoolType -> 
	(ABoolOp (op, e1, e2)), BoolType
      | _ -> 
	printError 2 " logical connective type mismatch "

let arithOp op p1 p2 = 
    let (e1,t1) = p1 in 
    let (e2,t2) = p2 in
    match (t1,t2) with
      | RealType, RealType -> 
	( AArithOp (op, e1, e2)), RealType
      | _ -> 
	printError 2 " arithmetic operator  type mismatch "
	
%}


%token <string> TNum TString TName 
%token <bool> TBool
%token TKwdNum TKwdBool TKwdString TKwdIn TKwdInput TKwdParam TKwdUserInput TKwdSimParam TColon TKwdEnum TKwdUnit TKwdEq
%token TVert TKwdAssert TKwdAssume TKwdDefined TKwdClock TKwdFreeze TKwdUnFreeze TKwdVar TKwdComponent TKwdNondet TKwdDot TAssign TKwdWhere
%token TPlus TMinus TDiv TStar TEq TSemiColon TComma TAnd TOr TImpl TArrow TNot TOpenRound TCloseRound TOpenCurly TCloseCurly
%token TOpenSquare TCloseSquare TLt TLeq TGt TGeq TNeq TKwdEnd  TKwdInit
%token TKwdStep TKwdMax TKwdWhen

%right  TAssign
%left   TOr
%left   TAnd
%left   TImpl
%left   TEq TNeq
%left   TGeq TLeq
%left   TGt TLt
%left   TPlus TMinus
%left   TStar TDiv
%left   TNot 
%right  UMINUS
%left   TOpenRound TCloseRound TOpenSquare TCloseSquare TKwdDot 

%type <unit> spec

%start spec


%%
spec: body TKwdEnd {}

body: body paramSpec {}
| body assertSpec {}
| body assumeSpec {}
| body componentSpec {}
| body optSpec {}
| /* -- EMPTY-- */ {}
;

/*---  | body variableDecl{ }
| body clockDecl{}
--*/

optSpec: TKwdMax expr TKwdIn  TOpenSquare num TComma num TCloseSquare TKwdStep num  whenExpr TSemiColon {
  let (e,eType) = $2 in 
  let l = $5 in 
  let u = $7 in 
  let s = $10 in 
  let c = $11 in 
    match eType with 
      | RealType ->
	  if (l >= u) 
	  then printError 5 "invalid range in max objective";
	  if (s <= 0.0)
	  then printError 10 "invalid step parameter in max objective";
	  ( addMaxObj e l u s c)
      | _ -> 
	  printError 2 "Type of expression must be real in a max objective"
}
;

whenExpr: {
  AConst(BoolConst true)
} |  TKwdWhen expr {
  let (e,eType) = $2 in 
    match eType with 
      | BoolType -> e
      | _ -> printError 2 "Boolean expression expected"
}
;

paramSpec: TKwdParam TLt paramTypeSpec TGt TName TColon typeName rangeSpec TSemiColon {
  match $8 with
    | RangeSpecNone -> 
      ( (*-- Type of parameter can be anything --*)
	match $7 with 
	  | EnumType 
	  | RealType 
	  | BoolType
	  | StringType -> declareParam $5 $3 $7 RangeSpecNone
	  | _ -> assert false
      )
    | RangeSpecInterval(l,u) -> 
      (
	if ($7 != RealType) 
	then printError 5 "Interval must be associated with a real-valued parameter";
	declareParam $5 $3 RealType $8
      )

    | RangeSpecEnum eList -> 
      (
	if ($7 != EnumType)
	then printError 5 "Enum list must be associated with an enum-typed parameter";
	declareParam $5 $3 (EnumType) $8
      )

    | RangeSpecEq (NumConst _ ) -> 
      ( 
	if ($7 != RealType)
	then printError 5 "Type mismatch for range specification";
	declareParam $5 $3 RealType $8
      )
    | RangeSpecEq (EnumConst _) -> 
      (
	if ($7 != EnumType)
	then printError 5 "Type mismatch for range specification";
	declareParam $5 $3 EnumType $8
      )
    | RangeSpecEq (BoolConst _) -> 
      (
	if ($7 != BoolType)
	then printError 5 "Type mismatch for range specification";
	declareParam $5 $3 BoolType $8
      )
    | _ -> assert false
} 
;

paramTypeSpec: TKwdUserInput {
  true
} 
| TKwdSimParam {
  false
};

typeName: TKwdNum {
  RealType
} 
| TKwdEnum {
  EnumType
} 
| TKwdBool {
  BoolType
} 
| TKwdString {
  StringType
}
;

rangeSpec: /* EMPTY */
  { RangeSpecNone }
|  TKwdIn intvlSpec {
  $2
}
| TKwdEq constant {
  let (e,eTyp) = $2 in 
  match e with 
    | (AConst ( c)) -> (RangeSpecEq c)
    | _ -> assert false
}
| TKwdEq identifier {
  match $2 with 
    | DeclVar vi -> ( printError 2 ("Identifier "^vi.vName^" used where we expected an enum constant") )
    | DeclEnum str -> (RangeSpecEq (EnumConst str))
    | DeclParam pI -> (printError 2 ("Parameter "^pI.pName^" used where we expected an enum constant"))
}
| TKwdIn enumSpec {
  let iList = declareNewEnumSymbols $2 in 
  RangeSpecEnum(iList)
}
;

intvlSpec: TOpenSquare TNum TComma TNum TCloseSquare {
  let l = float_of_string $2 in
  let u = float_of_string $4 in 
  if ( l > u) 
  then 
    printError 2 "Bad interval specifier";
  RangeSpecInterval(l,u)  
}
;

enumSpec: TOpenCurly identifierList TCloseCurly {
  $2
}
;

identifierList:  TName { [$1] }
| identifierList TComma TName { $3::$1 }
;

assumeSpec: TKwdAssume expr TSemiColon {
  let (e,eType) = $2 in 
  if not (eType = BoolType) 
  then printError 2 "Assumption over non-boolean expression";
  addAssumeSpec e
}


assertSpec: TKwdAssert expr TSemiColon {
  let (e,eType) = $2 in 
  ( match eType with 
    | BoolType -> ()
    | _ -> (printError 2 "Assertion over non-boolean expression")
  );
  addAssertSpec  e
};

expr: identifier {
  match $1 with 
    | DeclVar vi -> (AVar vi), vi.vType
    | DeclEnum (str) -> (AConst ( EnumConst(str) )), (EnumType)
    | DeclParam pI -> (AConst (ParamConst pI)), pI.pType
    | _ -> assert false
}
| constant {
  $1
}
| TKwdDefined TOpenRound identifier TCloseRound {
  match $3 with 
    | DeclParam pI ->   (ADefined pI), BoolType
    | _ -> printError 3 "not sure defined when applied to nonparameter"
}
| expr TEq expr {
  let (e1,t1) = $1 in 
  let (e2,t2) = $3 in 
  if not (t1 = t2) then 
    printError 2 "type mismatch in equality expression";
  ( ARelOp (AEq, e1, e2)), BoolType
}
| expr TNeq expr {
  let (e1,t1) = $1 in 
  let (e2,t2) = $3 in 
  if not (t1 = t2) then 
    printError 2 "type mismatch in disequality expression";
  (ARelOp (ANeq, e1, e2)), BoolType
}
| expr TAnd expr {
  boolOp AAnd $1 $3
}
| expr TOr expr {
  boolOp AOr $1 $3
}
| expr TPlus expr {
  arithOp APlus $1 $3
}
| expr TMinus expr {
  arithOp AMinus $1 $3
}
| expr TStar expr {
  arithOp AStar $1 $3

}
| expr TDiv expr {
  arithOp ADiv $1 $3
}
| expr TGeq expr { 
  relOp AGeq $1 $3
}
| expr TLeq expr {
  relOp AGeq $3 $1
}
| expr TGt expr {
  relOp AGt $1 $3
}
| expr TLt expr {
  relOp AGt $3 $1
}
| expr TImpl expr {
  boolOp AImpl $1 $3
}
| TOpenRound expr TCloseRound { $2 }
| TMinus expr %prec UMINUS { let (e,t) = $2 in 
			     if (t = RealType) 
			     then (ANeg e), RealType
			     else printError 1 "negation applied to non-arithmetic expression"
}
| TNot expr {
  let (e,t) = $2 in 
  if not (isBoolType t) 
  then  printError 1 "logical negation applied to non-Boolean expression";
  (ANot e), BoolType
}
;

componentSpec: TKwdComponent TName TOpenCurly componentBody TCloseCurly TSemiColon {
  let (declHash, iExpr, trsList) = $4 in 
  addComponent $2 declHash iExpr trsList

} ;

componentBody: varDecls initSpec trsSpec {
  ($1,$2,$3)
} ;

variableDecl:  TKwdVar  TName TColon typeName rangeSpec TSemiColon{
  let vi = declareVariable $2 $4 $5 in 
  vi
};

clockDecl: TKwdClock TName  rangeSpec TSemiColon{
  checkRangeTypeMatch $2 RealType $3;
  declareClock $2  $3
}
;
varDecls:  /*EMPTY*/ { Inthash.create 1000 }
| varDecls variableDecl {
  let vi = $2 in 
  Inthash.replace $1 vi.vID vi;
  $1
} 
| varDecls TKwdNondet TName TColon typeName rangeSpec TSemiColon {
  checkRangeTypeMatch $3 $5 $6;
  let vi = declareNondet $3 $5 $6 in 
  Inthash.replace $1 vi.vID vi;
  $1
}
| varDecls clockDecl {
  let vi = $2 in 
  Inthash.replace $1 vi.vID vi;
  $1
}
;

initSpec: TKwdInit expr TSemiColon {
  let (e,eT) = $2 in 
  if not (eT = BoolType)
  then printError 2 "Initializer spec must be a boolean";
  e
}
;

trsSpec:  TVert trs { $2 }
| TVert trs trsSpec {
  $2@$3
}
;

trs: expr TArrow TOpenCurly assignList TCloseCurly {
  let h,fList,ufList = $4 in 
  let (e,eType) = $1 in 
  if not (isBoolType eType)
  then 
    printError 1 "Transition guard should be a Boolean assertion. ";
  [ 
    makeTrs e h fList ufList 
  ]
}

| expr TArrow TOpenSquare trsSpec TCloseSquare {
  let tList = $4 in 
  let (e,eType) = $1 in 
  if not (isBoolType eType)
  then 
    printError 1 "Transition guard should be a Boolean assertion. ";
  List.iter (fun t -> (t.tGuard <- (ABoolOp( AAnd, e, t.tGuard) ) )) tList;
  tList
}

;
assignList: /* EMPTY */ {
  (Inthash.create 100), [], [] 
}
| assignList TKwdFreeze TOpenRound identifier TCloseRound TSemiColon {
  match $4 with 
    | DeclVar vi -> begin
      if (not vi.isClock) 
      then (printError 4 "unfreeze on a non-clock variable");
      let (h,freezeList, unfreezeList)=$1 in 
      (h, (vi::freezeList), unfreezeList)
    end
    | _ -> printError 4 "expected identifier, but encountered parameter/enum "
}
| assignList TKwdUnFreeze TOpenRound identifier TCloseRound TSemiColon{
  match $4 with 
    | DeclVar vi -> begin
      if (not vi.isClock) 
      then (printError 4 "unfreeze on a non-clock variable");
      let (h,fList, ufList)=$1 in 
      (h, fList, (vi::ufList))
    end
    | _ -> printError 4 "expected identifier, but encountered parameter/enum"

}
| assignList identifier TAssign expr TSemiColon  {
  let (e,eT) = $4 in 
  match $2 with 
    | DeclVar vi -> begin
      ( 
	if  not (typeCheckAssign vi $4) 
	then  printError 1 "Type mismatch in assignment"
      );
      let (h,fList, ufList)=$1 
      in 
      ( if (Inthash.mem h vi.vID)
	then 
	  printError 1 "Variable already assigned in transition" 
      );
      Inthash.replace h vi.vID e;
      (h,fList,ufList)
    end
    | _ -> printError 2 "expected identifier, but encountered parameter/enum"

}
;

identifier: TName {
  (* The name can be that of a previously declared variable or an enum *)
  try 
    resolveIdentifier $1
  with 
    | Not_found -> printError 1 ("unknown variable:"^$1)
}
;
constant: num {
  ( AConst ( NumConst(  $1) ) ), RealType
}
| TBool {
  ( AConst ( 
    BoolConst $1
    )), BoolType
}
| TString {
  (AConst (StringConst $1)), StringType
}

;

num: TNum {
  float_of_string $1
}
;
%%
