(*---
  Constant fold the AST to get rid of parameters that have been defined to a constant.
  ---*)


open MyUtils

module C=CompAst


let rec cFoldExpr gInfo e =
  let f e' = cFoldExpr gInfo e' in  
    match e with 
    | C.ABoolOp (b, e1,e2) -> C.ABoolOp (b, (f e1) , (f e2))
    | C.ANot e -> C.ANot (f e)
    | C.ARelOp (r, e1, e2) -> C.ARelOp (r , (f e1), (f e2))
    | C.AArithOp (a, e1, e2) -> C.AArithOp (a, (f e1), (f e2))
    | C.ANeg e -> C.ANeg (f e)
    | C.AConst (C.ParamConst pI) -> 
	begin
	  match pI.C.pRange with 
	    | C.RangeSpecEq cI -> C.AConst cI
	    | _ -> e
	end
    | C.ADefined pI ->
	begin
	  match pI.C.pRange with
	    | C.RangeSpecEq cI -> C.AConst (C.BoolConst true)
	    | _ -> e
	end
    | _ -> e

let cFoldTrs gInfo trs = 
  trs.C.tGuard <- (cFoldExpr gInfo trs.C.tGuard);
  let newHash = Inthash.create 100 in 
    Inthash.iter (fun i e -> 
		    let e' = cFoldExpr gInfo e in 
		      (Inthash.replace newHash i e')
		 )  trs.C.tUpdates;
    trs.C.tUpdates <- newHash




let cFoldComp gInfo c = 
  c.C.cInit <- cFoldExpr gInfo c.C.cInit;
  List.iter (cFoldTrs gInfo) c.C.cTransitions

let cFoldMain gInfo = 
  let hPrp p = { C.prpID = p.C.prpID; C.pExpr = (cFoldExpr gInfo p.C.pExpr); C.pViolated = false } in 
  gInfo.C.gProps <- List.map hPrp gInfo.C.gProps;
  List.iter (cFoldComp gInfo) gInfo.C.gComponents


