let maxDepth = ref 100
  
let simOnly = ref false
let numSim = ref 200 
 
let parseFile str strParams =
  try
    (Printf.printf "Parsing file %s \n" str;
     flush stdout;
     let lexbuf = Lexing.from_channel (open_in str)
     in
       (AutoParser.spec AutoLexer.autoLexer lexbuf;
        if (String.length strParams) > 0
        then
          (let oc = open_in strParams
           in (CompAst.readParametersFromFile oc; close_in oc))
        else ();
        CFold.cFoldMain CompAst.gInfo;
        if !simOnly
        then (
	  TraceExtraction.exploreTracesForMaxMin CompAst.gInfo !numSim !maxDepth  	)
        else
          (* let oc = open_out (str^".output.xml") in                              *)
          (* XmlAstPrinter.xmlPrintGlobinfo oc CompAst.gInfo;                      *)
          (let eInfo = SmtEncoder.initializeEncoding CompAst.gInfo
           in
             try
               (for depth = 1 to !maxDepth do
                  Printf.printf "--- Depth %d -- \n" depth;
                  flush stdout;
                  SmtEncoder.increaseEncodingDepth CompAst.gInfo eInfo;
                  SmtEncoder.checkPropertyViolation CompAst.gInfo eInfo;
                  (* SmtEncoder.encodeAsserts CompAst.gInfo eInfo *)
                  SmtEncoder.handleMaximization eInfo CompAst.gInfo
                done;
                SmtEncoder.extractValuesAtDepth CompAst.gInfo eInfo;
                CompAst.printMaxValues () )
(*                let pTimes = Unix.times ()
                in
                  Printf.printf "User: %f System: %f \n"
                    pTimes.Unix.tms_utime pTimes.Unix.tms_stime) *)
             with
             | SmtEncoder.PropertyFailed ->
                 (Printf.printf "Property violation discovered \n";
                  SmtEncoder.extractValuesAtDepth CompAst.gInfo eInfo)
             | SmtEncoder.AllPropsResolved ->
                 Printf.printf " -- All properties have been resolved --- ")))
  with | End_of_file -> ()
  
let fileList = ref []
  
let usageMsg = "usage: autoParse [-d depth] modelfile paramsFile "
  
let _ =
  try
    (Arg.parse
       [ ("-d", (Arg.Set_int maxDepth), " depth of trace ");
         ("-sim", (Arg.Set simOnly), " Run the simulator");
         ("-numSim", (Arg.Set_int numSim), " number of simulations") ]
       (fun s -> fileList := !fileList @ [ s ]) usageMsg;
     if (List.length !fileList) <= 0
     then Printf.fprintf stderr "%s \n" usageMsg
     else
       (if (List.length !fileList) <= 1
        then parseFile (List.hd !fileList) ""
        else parseFile (List.nth !fileList 0) (List.nth !fileList 1);
        Printf.fprintf stdout " --> Okey Dokey <--- \n"))
  with
  | Parsing.Parse_error ->
      Printf.fprintf stderr "Parse Error @line %d \n" !AutoLexer.lineNum
  | CompAst.SemanticError str ->
      Printf.fprintf stderr "Parsing failed. Semantic Error: %s \n" str
  
