module C=CompAst
module Y=YicesInterface

type context = Y.yices_context
type expr = Y.yices_expr

type model = Y.yices_model


let createContext = Y.mk_context 
let delContext = Y.del_context 

let andExpr ctx e1 e2 = Y.mk_and ctx [| e1; e2 |]

let orExpr ctx e1 e2 = Y.mk_or ctx [| e1; e2 |]

let implExpr ctx e1 e2 = Y.mk_or ctx [| (Y.mk_not ctx e1); e2 |]

let negateExpr ctx e = Y.mk_not ctx e

let geqExpr ctx e1 e2 = Y.mk_ge ctx e1 e2


let gtExpr ctx e1 e2  = Y.mk_gt ctx e1 e2

let eqExpr ctx e1 e2 = Y.mk_eq ctx e1 e2

let neqExpr ctx e1 e2 = Y.mk_diseq ctx e1 e2


let plusExpr ctx e1 e2 = Y.add ctx e1 e2

let minusExpr ctx e1 e2 = Y.sub ctx e1 e2

let starExpr ctx e1 e2 = Y.mul ctx e1 e2

let divExpr ctx e1 e2 = assert false (* Not supported by yices yet *)

let boolConstant ctx b = 
  if (b) 
  then Y.mk_true ctx 
  else Y.mk_false ctx

let floatConstant ctx f = Y.mk_num_from_string ctx (Printf.sprintf "%d/%d" (int_of_float (f *. 10000.0)) 10000)

let intConstant ctx i = Y.mk_num ctx i 

let createVariable ctx vName vType = 
  match vType with 
    | C.BoolType -> Y.mk_bool_var ctx vName
    | C.RealType -> Y.mk_real_var ctx vName
    | C.EnumType -> Y.mk_real_var ctx vName
    | C.StringType -> assert false

let assertExpr ctx e =   Y.assert_expr ctx e

let dumpContext ctx = Y.dump_context ctx

let ppExpr ctx e = Y.pp_expr e

type outcome = | SAT 
	       | UNSAT
	       | FAIL

let check ctx = 
  ignore (Y.check_ctx ctx);
  if (Y.inconsistent ctx) 
  then UNSAT
  else SAT


let getModel ctx = 
  Y.get_model ctx


let evalBoolVar sModel e = let i = Y.eval_bool_var sModel e in 
  if ( i == 0) then false else true

let evalIntVar  = Y.eval_int_var

let evalRealVar sModel e = Printf.sprintf "%f" (Y.eval_real_var sModel e)


let push = Y.push

let pop = Y.pop

