module C=CompAst


type context = Z3.context
type expr = Z3.ast
type model = Z3.context * Z3.model

let config = ref None

let createContext () = 
  let cfg = Z3.mk_config () in 
  Z3.set_param_value cfg "MODEL" "true";
  config := Some cfg;
  Z3.mk_context cfg

let delContext ctx = 
  Z3.del_context ctx ;
  match !config with 
    | Some cfg -> Z3.del_config cfg ; config := None 
    | None -> ()


let delModel (ctx,mdl) = 
  Z3.del_model ctx mdl



let andExpr ctx e1 e2 = Z3.mk_and ctx [| e1; e2 |]

let orExpr ctx e1 e2 = Z3.mk_or ctx [| e1; e2 |]

let implExpr ctx e1 e2 = Z3.mk_implies ctx e1  e2

let negateExpr ctx e = Z3.mk_not ctx e

let geqExpr ctx e1 e2 = Z3.mk_ge ctx e1 e2


let gtExpr ctx e1 e2  = Z3.mk_gt ctx e1 e2

let eqExpr ctx e1 e2 = Z3.mk_eq ctx e1 e2

let neqExpr ctx e1 e2 = Z3.mk_distinct ctx [| e1;  e2 |]


let plusExpr ctx e1 e2 = Z3.mk_add ctx [| e1 ; e2 |]

let minusExpr ctx e1 e2 = Z3.mk_sub ctx [| e1; e2 |]

let starExpr ctx e1 e2 = Z3.mk_mul ctx [| e1 ; e2 |]

let divExpr ctx e1 e2 = Z3.mk_div ctx e1 e2

let boolConstant ctx b = 
  if (b) 
  then Z3.mk_true ctx 
  else Z3.mk_false ctx

let floatConstant ctx f = Z3.mk_real ctx (int_of_float (f *. 100000.0)) 100000

let intConstant ctx i = Z3.mk_int ctx i (Z3.mk_int_sort ctx)

let mk_var ctx name ty = Z3.mk_const ctx (Z3.mk_string_symbol ctx name) ty;;

let createVariable ctx vName vType = 
  match vType with 
    | C.BoolType -> mk_var ctx vName (Z3.mk_bool_sort ctx) 
    | C.RealType -> mk_var ctx vName (Z3.mk_real_sort ctx)
    | C.EnumType -> mk_var ctx vName (Z3.mk_int_sort ctx)
    | C.StringType -> assert false

let assertExpr ctx e =   Z3.assert_cnstr ctx e

let dumpContext ctx = Printf.printf "CTX: %s" (Z3.context_to_string ctx)

let ppExpr ctx e = Printf.printf "-> %s \n"(Z3.ast_to_string ctx e)

type outcome = | SAT 
	       | UNSAT
	       | FAIL

let check ctx = 
  match Z3.check ctx with 
    | Z3.L_FALSE -> UNSAT
    | Z3.L_TRUE -> SAT
    | Z3.L_UNDEF -> FAIL


let getModel ctx = 
  let (r,m) = Z3.check_and_get_model ctx in 
  match r with 
    | Z3.L_TRUE -> (ctx,m)
    | _ -> assert false


let evalBoolVar (ctx,sModel) e = 
  let (res,v) = Z3.eval ctx sModel e in 
  assert(res);
  match Z3.get_bool_value ctx v with
    | Z3.L_TRUE -> true
    | Z3.L_FALSE -> false
    | Z3.L_UNDEF -> false

let evalIntVar  (ctx,sModel) e = 
  let (res,v) = Z3.eval ctx sModel e in 
  assert(res);
  let (res1,vVal) = Z3.get_numeral_int ctx v in
  assert(res1);
  vVal

let evalRealVar (ctx,sModel) e = 
  let (res,v) = Z3.eval ctx sModel e in 
  assert(res);
  Z3.get_numeral_string ctx v



let push = Z3.push

let pop ctx = Z3.pop ctx 1

