(*-- 
  Extract a Trace from the SMT encoding 
  --*)

(*---
  1. Extract values of all parameters.
  2. Extract the following values at every time step:
        1. Values of all variables at the depth.
        2. For each clock, is it frozen or not.
        3. Which transition is taken for each component 
  ---*)
open MyUtils
module C = CompAst
module AS = AstSimulator



(*-- Represent max/min values of an opt. expression at each step --*)
    
type maxmin = {
  nSteps: int; (*-- Number of steps performed in each simulation --*)
  mutable nSims: int; (*-- Number of simulations carried out --*)
  maxVals: float array; (*-- Maximum values for the optm. expression at each time step --*)
  minVals: float array; (*-- Minimum values for optm. expression at each time step --*)
}


let initMaxMin n loVal hiVal = {
  nSteps =  n;
  nSims = 0;
  maxVals = Array.create n loVal;
  minVals = Array.create n hiVal
}

let printMinMaxVals mm = 
  Printf.printf " Step Num, Min, Max \n";
  for i = 0 to (mm.nSteps -1)
  do
    Printf.printf " %d,  %f, %f \n" i  mm.minVals.(i) mm.maxVals.(i)
  done

let exploreTracesForMaxMin gInfo nSims nStepsPerSim = 
(*-- 1. Perform nSims many simulations.  2. For each simulation,
  check if max/min at each time point needs to be updated.  --*)
  if (List.length gInfo.C.gMax) <= 0
  then ( 
    Printf.fprintf stderr "Nothing to max/min. Please consider specifying an expression for simulation . \n";
    AS.simulate gInfo nStepsPerSim
  ) else (
    let gObj = List.hd gInfo.C.gMax in 
    let gExpr = gObj.C.mExpr in 
    let mm = initMaxMin nStepsPerSim gObj.C.mLo gObj.C.mHi in
    for i = 0 to nSims
    do
      let stArray = AS.simAndGenerateTrace gInfo nStepsPerSim in 
      AS.computeObjectiveValue gInfo gExpr stArray mm.maxVals mm.minVals ;
      mm.nSims <- (i+1)
    done;
    printMinMaxVals mm
  )

